from functools import wraps
from inspect import getfullargspec
from typing import get_type_hints

# Each port has two gpio pins associated with it
# Default pin for each class is "1"
# Convert Port for use with standard BCM pins
Port = {
    "A0": {"1": 0, "2": 1},
    "A1": {"1": 2, "2": 3},
    "A2": {"1": 4, "2": 5},
    "A3": {"1": 6, "2": 7},
    "D0": {"1": 22, "2": 23},
    "D1": {"1": 24, "2": 25},
    "D2": {"1": 26, "2": 27},
    "D3": {"1": 5, "2": 6},
    "D4": {"1": 7, "2": 8},
    "D5": {"1": 10, "2": 11},
    "D6": {"1": 12, "2": 13},
    "D7": {"1": 15, "2": 14},
}


def get_pin_for_port(port_name, pin_number=1):
    assert port_name in Port
    assert pin_number in (1, 2)

    return Port[port_name][str(pin_number)]


def validate_input(obj, **kwargs):
    hints = get_type_hints(obj)
    for attr_name, attr_type in hints.items():
        if attr_name == "return":
            continue

        attr_type_arr = [attr_type]
        if hasattr(attr_type, "__args__"):
            attr_type_arr = attr_type.__args__

        has_errors = True
        for attr_subtype in attr_type_arr:
            # if we use a default value on the method, it won't be in kwargs
            if attr_name not in kwargs or isinstance(kwargs[attr_name], attr_subtype):
                has_errors = False
                break
        if has_errors:
            raise TypeError(f"Argument '{attr_name}' is not of type {attr_type}")


def type_check(decorator):
    @wraps(decorator)
    def wrapped_decorator(*args, **kwargs):
        # translate *args into **kwargs
        func_args = getfullargspec(decorator)[0]
        kwargs.update(dict(zip(func_args, args)))

        validate_input(decorator, **kwargs)
        return decorator(**kwargs)

    return wrapped_decorator



from typing import Union

from numpy import interp

from pitop.common.bitwise_ops import split_into_bytes

from .common import type_check
from .common.servo_motor_registers import (
    ServoControlRegisters,
    ServoMotorSetup,
    ServoRegisterTypes,
)
from .plate_interface import PlateInterface


class ServoHardwareSpecs:
    PWM_FREQUENCY = 60
    ANGLE_RANGE = 180
    SPEED_RANGE = 100
    MIN_PULSE_WIDTH_MICRO_S = 911
    MAX_PULSE_WIDTH_MICRO_S = 2165
    DUTY_REGISTER_RANGE = 4095


class ServoController:
    """Class used to read/write servo motor registers from the MCU."""

    def __init__(self, port: str):
        if port not in ServoControlRegisters.__members__:
            raise Exception(
                "Invalid port. Servo motors must be connected to ports S0-S3"
            )

        self.registers = ServoControlRegisters[port].value
        self._mcu_device = PlateInterface().get_device_mcu()

        self.__lower_duty_cycle = 0
        self.__upper_duty_cycle = 0

        self.set_pwm_frequency(ServoHardwareSpecs.PWM_FREQUENCY)
        self.set_min_pulse_width(ServoHardwareSpecs.MIN_PULSE_WIDTH_MICRO_S)
        self.set_max_pulse_width(ServoHardwareSpecs.MAX_PULSE_WIDTH_MICRO_S)

    @type_check
    def set_min_pulse_width(self, min_width_us: int) -> None:
        self._mcu_device.write_word(
            ServoMotorSetup.REGISTER_MIN_PULSE_WIDTH,
            min_width_us,
            signed=False,
            little_endian=True,
        )

        self.__lower_duty_cycle = ServoHardwareSpecs.DUTY_REGISTER_RANGE * (
            (min_width_us * 1e-6) * self.pwm_frequency()
        )
        self.__lower_duty_cycle = int(round(self.__lower_duty_cycle))

    @type_check
    def set_max_pulse_width(self, max_width_us: int) -> None:
        self._mcu_device.write_word(
            ServoMotorSetup.REGISTER_MAX_PULSE_WIDTH,
            max_width_us,
            signed=False,
            little_endian=True,
        )

        self.__upper_duty_cycle = ServoHardwareSpecs.DUTY_REGISTER_RANGE * (
            (max_width_us * 1e-6) * self.pwm_frequency()
        )
        self.__upper_duty_cycle = int(round(self.__upper_duty_cycle))

    @type_check
    def set_pwm_frequency(self, frequency: int) -> None:
        self._mcu_device.write_byte(ServoMotorSetup.REGISTER_PWM_FREQUENCY, frequency)

    def pwm_frequency(self) -> int:
        return self._mcu_device.read_unsigned_byte(
            ServoMotorSetup.REGISTER_PWM_FREQUENCY
        )

    def get_current_angle_and_speed(self):
        duty_cycle_and_speed = self._mcu_device.read_n_signed_bytes(
            self.registers[ServoRegisterTypes.ANGLE_AND_SPEED],
            number_of_bytes=4,
            little_endian=True,
        )

        angle_speed_bytes = split_into_bytes(
            duty_cycle_and_speed, no_of_bytes=4, little_endian=True, signed=True
        )

        duty_cycle = int.from_bytes(
            angle_speed_bytes[0:2], byteorder="little", signed=True
        )

        angle = int(
            round(
                interp(
                    duty_cycle,
                    [self.__lower_duty_cycle, self.__upper_duty_cycle],
                    [
                        -ServoHardwareSpecs.ANGLE_RANGE / 2,
                        ServoHardwareSpecs.ANGLE_RANGE / 2,
                    ],
                )
            )
        )

        speed = (
            int.from_bytes(angle_speed_bytes[2:4], byteorder="little", signed=True)
            / 10.0
        )

        return angle, speed

    @type_check
    def set_target_angle(
        self, angle: Union[int, float], speed: Union[int, float] = 50.0
    ):
        mapped_duty_cycle = interp(
            angle,
            [-ServoHardwareSpecs.ANGLE_RANGE / 2, ServoHardwareSpecs.ANGLE_RANGE / 2],
            [self.__lower_duty_cycle, self.__upper_duty_cycle],
        )
        mapped_duty_cycle = int(round(mapped_duty_cycle))

        mapped_speed = int(round(speed * 10))

        list_to_send = split_into_bytes(
            mapped_duty_cycle, 2, signed=True, little_endian=True
        )
        list_to_send += split_into_bytes(
            mapped_speed, 2, signed=True, little_endian=True
        )

        self._mcu_device.write_n_bytes(
            self.registers[ServoRegisterTypes.ANGLE_AND_SPEED], list_to_send
        )

    def get_acceleration_mode(self):
        return self._mcu_device.read_unsigned_byte(
            self.registers[ServoRegisterTypes.ACC_MODE]
        )

    @type_check
    def set_acceleration_mode(self, mode: int):
        """Set the acceleration mode of the servo.

        When set to 0, the servo will run in 'normal mode', where speed
        is constant. When set to 1, the servo will run in 'smoothed
        mode', where a linear acceleration profile is used to ramp speed
        at start and end of cycle.
        """
        self._mcu_device.write_byte(self.registers[ServoRegisterTypes.ACC_MODE], mode)



from .adc_base import ADCBase


class LightSensor(ADCBase):
    """Encapsulates the behaviour of a light sensor module.

    A simple analogue photo transistor is used to detect the intensity of the light striking
    the sensor. The component contains a photoresistor which detects light intensity. The
    resistance decreases as light intensity increases; thus the brighter the light, the
    higher the voltage.

    Uses an Analog-to-Digital Converter (ADC) to turn the analog reading from the sensor
    into a digital value.

    By default, the sensor uses 3 samples to report a :attr:`~.LightSensor.reading`, which takes around 0.5s.
    This can be changed by modifying the parameter :attr:`~.LightSensor.number_of_samples` in the constructor.

    :param str port_name: The ID for the port to which this component is connected
    :param str number_of_samples: Amount of sensor samples used to report a :attr:`~.LightSensor.reading`. Defaults to 3.
    :param str name: Component name, defaults to `light_sensor`. Used to access this component when added to a :class:`pitop.Pitop` object.
    """

    def __init__(
        self, port_name, pin_number=1, name="light_sensor", number_of_samples=3
    ):
        ADCBase.__init__(
            self,
            port_name=port_name,
            pin_number=pin_number,
            name=name,
            number_of_samples=number_of_samples,
        )

    @property
    def reading(self):
        """Take a reading from the sensor.

        :return: A value representing the amount of light striking the
            sensor at the current time from 0 to 999.
        :rtype: float
        """
        return int(self.read())

    @property
    def value(self):
        """Get a simple binary value based on a reading from the device.

        :return: 1 if the sensor is detecting any light, 0 otherwise
        :rtype: integer
        """
        if self.reading > 0:
            return 1
        else:
            return 0

    @property
    def own_state(self):
        return {
            "value": lambda: self.value,
            "reading": lambda: self.reading,
        }



import atexit
import time
from math import floor, pi

from pitop.core.mixins import Recreatable, Stateful

from .encoder_motor_controller import EncoderMotorController
from .parameters import BrakingType, Direction, ForwardDirection


class EncoderMotor(Stateful, Recreatable):
    """Represents a pi-top motor encoder component.

    Note that pi-top motor encoders use a built-in closed-loop control system, that feeds the readings
    from an encoder sensor to an PID controller. This controller will actively modify the motor's current to move at the desired
    speed or position, even if a load is applied to the shaft.

    This internal controller is used when moving the motor through :class:`set_target_rpm` or :class:`set_target_speed`
    methods, while using the :class:`set_power` method will make the motor work in open-loop, not using the controller.

    .. note::
        Note that some methods allow to use distance and speed settings in meters and meters per second. These will only make
        sense when using a wheel attached to the shaft of the motor.

        The conversions between angle, rotations and RPM used by the motor to meters and meters/second are performed considering
        the :data:`wheel_diameter` parameter. This parameter defaults to the diameter of the wheel included with MMK.
        If a wheel of different dimmensions is attached to the motor, you'll need to measure it's diameter, in order for these
        methods to work properly.

    :type port_name: str
    :param port_name:
        The ID for the port to which this component is connected.

    :type forward_direction: ForwardDirection
    :param forward_direction:
        The type of rotation of the motor shaft that corresponds to forward motion.

    :type braking_type: BrakingType
    :param braking_type:
        The braking type of the motor. Defaults to coast.

    :type wheel_diameter: int or float
    :param wheel_diameter:
        The diameter of the wheel attached to the motor.

    :type name: str
    :param name:
        Component name, defaults to `encoder_motor`. Used to access this component when added to a :class:`pitop.Pitop` object.
    """

    MMK_STANDARD_GEAR_RATIO = 41.8
    MAX_DC_MOTOR_RPM = 4800

    def __init__(
        self,
        port_name,
        forward_direction,
        braking_type=BrakingType.BRAKE,
        wheel_diameter=0.070,
        name="encoder_motor",
    ):
        self.name = name
        self._pma_port = port_name

        if isinstance(braking_type, BrakingType):
            braking_type = braking_type.value

        self.__motor_core = EncoderMotorController(self._pma_port, braking_type)
        self.__forward_direction = forward_direction
        self.__wheel_diameter = wheel_diameter

        self.__prev_time_dist_cnt = time.time()
        self.__previous_reading_odometer = 0

        atexit.register(self.stop)

        Stateful.__init__(self)
        Recreatable.__init__(
            self,
            config_dict={
                "port_name": port_name,
                "name": name,
                "forward_direction": lambda: self.forward_direction,
                "braking_type": lambda: self.braking_type,
                "wheel_diameter": lambda: self.wheel_diameter,
            },
        )

    @property
    def own_state(self):
        return {
            "current_rpm": lambda: self.current_rpm,
            "current_speed": lambda: self.current_speed,
            "distance": lambda: self.distance,
            "wheel_diameter": lambda: self.wheel_diameter,
            "forward_direction": lambda: self.forward_direction,
            "braking_type": lambda: self.braking_type,
        }

    @property
    def forward_direction(self):
        """Represents the forward direction setting used by the motor.

        Setting this property will determine on which direction the
        motor will turn whenever a movement in a particular direction is
        requested.

        :type forward_direction: ForwardDirection
        :param forward_direction: The direction that corresponds to
            forward motion.
        """
        return ForwardDirection(self.__forward_direction)

    @forward_direction.setter
    def forward_direction(self, forward_direction):
        self.__forward_direction = forward_direction

    @property
    def braking_type(self):
        """Returns the type of braking used by the motor when it's stopping
        after a movement.

        Setting this property will change the way the motor stops a movement:

        - :data:`BrakingType.COAST` will make the motor coast to a halt when stopped.
        - :data:`BrakingType.BRAKE` will cause the motor to actively brake when stopped.

        :type braking_type: BrakingType
        :param braking_type:
            The braking type of the motor.
        """
        return BrakingType(self.__motor_core.braking_type())

    @braking_type.setter
    def braking_type(self, braking_type):
        self.__motor_core.set_braking_type(braking_type.value)

    def set_power(self, power, direction=Direction.FORWARD):
        """Turn the motor on at the power level provided, in the range -1.0 to.

        +1.0, where:

        - 1.0: motor will turn with full power in the :data:`direction` provided as argument.
        - 0.0: motor will not move.
        - -1.0: motor will turn with full power in the direction contrary to :data:`direction`.

        .. warning::
            Setting a :data:`.power` value out of range will cause the method to raise an
            exception.

        :type power: int or float
        :param power:
            Motor power, in the range -1.0 to +1.0

        :type direction: Direction
        :param direction:
            Direction to rotate the motor
        """

        if not (-1.0 <= power <= 1.0):
            raise ValueError("Power value must be between -1.0 and +1.0 (inclusive)")

        power_mapping = int(round(power * 1000) * self.__forward_direction * direction)

        self.__motor_core.set_power(power_mapping)

    def power(self):
        """Get the current power of the motor.

        Returns a value from -1.0 to +1.0, assuming the user is
        controlling the motor using the :class:`set_power` method (motor
        is in control mode 0). If this is not the case, returns None.
        """

        power = self.__motor_core.power()
        if power:
            return power / 1000.0
        return None

    def set_target_rpm(
        self, target_rpm, direction=Direction.FORWARD, total_rotations=0.0
    ):
        """Run the motor at the specified :data:`.target_rpm` RPM.

        If desired, a number of full or partial rotations can also be set through the :data:`total_rotations`
        parameter. Once reached, the motor will stop. Setting :data:`total_rotations` to 0 will set the
        motor to run indefinitely until stopped.

        If the desired RPM setting cannot be achieved, :class:`torque_limited` will be set to :data:`True`
        and the motor will run at the maximum possible RPM it is capable of for the instantaneous torque.
        This means that if the torque lowers, then the RPM will continue to rise until it meets the
        desired level.

        Care needs to be taken here if you want to drive a vehicle forward in a straight line, as the motors
        are not guaranteed to spin at the same rate if they are torque-limited.

        .. warning::
            Setting a :data:`.target_rpm` higher than the maximum allowed will cause the
            method to throw an exception. To determine what the maximum possible target RPM for the motor
            is, use the :class:`max_rpm` method.

        :type target_rpm: int or float
        :param target_rpm:
            Desired RPM of output shaft

        :type direction: Direction
        :param direction:
            Direction to rotate the motor. Defaults to forward.

        :type total_rotations: int or float
        :param total_rotations:
            Total number of rotations to be execute. Set to 0 to run indefinitely.
        """

        if not (-self.max_rpm <= abs(target_rpm) <= self.max_rpm):
            raise ValueError(
                f"Target RPM value must be between {-self.max_rpm} and {self.max_rpm} (inclusive)"
            )

        dc_motor_rpm = int(
            round(target_rpm * self.MMK_STANDARD_GEAR_RATIO)
            * self.__forward_direction
            * direction
        )
        dc_motor_rotations = int(
            round(total_rotations * self.MMK_STANDARD_GEAR_RATIO)
            * self.__forward_direction
            * direction
        )

        if not (-self.MAX_DC_MOTOR_RPM <= dc_motor_rpm <= self.MAX_DC_MOTOR_RPM):
            raise ValueError(
                f"DC motor RPM value must be between {-self.MAX_DC_MOTOR_RPM} and {self.MAX_DC_MOTOR_RPM} (inclusive)"
            )

        if dc_motor_rotations == 0:
            self.__motor_core.set_rpm_control(dc_motor_rpm)
        else:
            dc_motor_rotation_counter = self.__motor_core.odometer()
            rotations_offset_to_send = int(
                dc_motor_rotations + dc_motor_rotation_counter
            )
            self.__motor_core.set_rpm_with_rotations(
                dc_motor_rpm, rotations_offset_to_send
            )

    def target_rpm(self):
        """Get the desired RPM of the motor output shaft, assuming the user is
        controlling the motor using :class:`set_target_rpm` (motor is in
        control mode 1).

        If this is not the case, returns None.
        """

        rpm = self.__motor_core.rpm_control()
        if rpm:
            return rpm / self.MMK_STANDARD_GEAR_RATIO * self.__forward_direction
        return None

    def stop(self):
        """Stop the motor in all circumstances."""
        self.__motor_core.stop()

    @property
    def current_rpm(self):
        """Returns the actual RPM currently being achieved at the output shaft,
        measured by the encoder sensor.

        This value might differ from the target RPM set through
        :class:`set_target_rpm`.
        """

        dc_motor_rpm_actual = self.__motor_core.tachometer() * self.__forward_direction
        output_shaft_rpm_actual = dc_motor_rpm_actual / self.MMK_STANDARD_GEAR_RATIO

        return output_shaft_rpm_actual

    @property
    def rotation_counter(self):
        """Returns the total or partial number of rotations performed by the
        motor shaft.

        Rotations will increment when moving forward, and decrement when
        moving backward. This value is a float with many decimal points
        of accuracy, so can be used to monitor even very small turns of
        the output shaft.
        """

        dc_motor_rotation_counter = (
            self.__motor_core.odometer() * self.__forward_direction
        )
        output_shaft_rotation_counter = round(
            dc_motor_rotation_counter / self.MMK_STANDARD_GEAR_RATIO, 1
        )

        return output_shaft_rotation_counter

    @property
    def torque_limited(self):
        """Check if the actual motor speed or RPM does not match the target
        speed or RPM.

        Returns a boolean value, :data:`True` if the motor is torque-
        limited and :data:`False` if it is not.
        """

        return False

    @property
    def max_rpm(self):
        """Returns the approximate maximum RPM capable given the motor and gear
        ratio."""

        return floor(self.MAX_DC_MOTOR_RPM / self.MMK_STANDARD_GEAR_RATIO)

    @property
    def wheel_diameter(self):
        """Represents the diameter of the wheel attached to the motor in
        meters.

        This parameter is important if using library functions to measure speed or distance, as these rely on
        knowing the diameter of the wheel in order to function correctly.
        Use the predefined pi-top wheel measurement, or define your own wheel size.

        .. note::
            The wheel diameter changes slightly when in contact with a surface due to rubber compression.

            For this application, the key measurement is the diameter when the wheel rests on a flat surface.
            Based on this, the default wheel diameter used is 0.070m.

        :type wheel_diameter: int or float
        :param wheel_diameter:
            Wheel diameter in meters.
        """
        return self.__wheel_diameter

    @wheel_diameter.setter
    def wheel_diameter(self, wheel_diameter):
        if wheel_diameter <= 0.0:
            raise ValueError("Wheel diameter must be higher than 0")

        self.__wheel_diameter = wheel_diameter

    @property
    def wheel_circumference(self):
        return self.wheel_diameter * pi

    def set_target_speed(self, target_speed, direction=Direction.FORWARD, distance=0.0):
        """Run the wheel at the specified target speed in meters per second.

        If desired, a :data:`distance` to travel can also be specified in meters, after which the motor
        will stop. Setting :data:`distance` to 0 will set the motor to run indefinitely until stopped.

        .. warning::
            Setting a :data:`.target_speed` higher than the maximum allowed will cause the
            method to throw an exception. To determine what the maximum possible target speed for the motor
            is, use the :class:`max_speed` method.

        .. note::
            Note that for this method to move the wheel the expected :data:`distance`, the correct
            :data:`wheel_diameter` value needs to be used.

        :type target_speed: int or float
        :param target_speed:
            Desired speed in m/s

        :type direction: Direction
        :param direction:
            Direction to rotate the motor. Defaults to forward.

        :type distance: int or float
        :param distance:
            Total distance to travel in m. Set to 0 to run indefinitely.
        """
        if not (-self.max_speed <= target_speed <= self.max_speed):
            raise ValueError(
                f"Wheel speed value must be between {-self.max_speed} and {self.max_speed} (inclusive)"
            )

        rpm = 60.0 * (target_speed / self.wheel_circumference)
        total_rotations = distance / self.wheel_circumference

        self.set_target_rpm(rpm, direction, total_rotations)

    def forward(self, target_speed, distance=0.0):
        """Run the wheel forward at the desired speed in meters per second.

        This method is a simple interface to move the motor that wraps a call to :data:`set_target_speed`,
        specifying the forward direction.

        If desired, a :data:`distance` to travel can also be specified in meters, after which the motor
        will stop. Setting :data:`distance` to 0 will set the motor to run indefinitely until stopped.

        .. note::
            Note that for this method to move the wheel the expected :data:`distance`, the correct
            :data:`wheel_circumference` value needs to be used.

        :type target_speed: int or float
        :param target_speed:
            Desired speed in m/s

        :type distance: int or float
        :param distance:
            Total distance to travel in m. Set to 0 to run indefinitely.
        """

        self.set_target_speed(target_speed, Direction.FORWARD, distance)

    def backward(self, target_speed, distance=0.0):
        """Run the wheel backwards at the desired speed in meters per second.

        This method is a simple interface to move the wheel that wraps a call to :data:`set_target_speed`,
        specifying the back direction.

        If desired, a :data:`distance` to travel can also be specified in meters, after which the motor
        will stop. Setting :data:`distance` to 0 will set the motor to run indefinitely until stopped.

        .. note::
            Note that for this method to move the wheel the expected :data:`distance`, the correct
            :data:`wheel_circumference` value needs to be used.

        :type target_speed: int or float
        :param target_speed:
            Desired speed in m/s

        :type distance: int or float
        :param distance:
            Total distance to travel in m. Set to 0 to run indefinitely.
        """

        self.set_target_speed(target_speed, Direction.BACK, distance)

    @property
    def current_speed(self):
        """Returns the speed currently being achieved by the motor in meters
        per second.

        This value may differ from the target speed set through
        :class:`set_target_speed`.
        """

        return (self.current_rpm / 60.0) * self.wheel_circumference

    @property
    def distance(self):
        """Returns the distance the wheel has travelled in meters.

        This value depends on the correct :data:`wheel_circumference`
        value being set.
        """

        return self.wheel_circumference * self.rotation_counter

    @property
    def max_speed(self):
        """The approximate maximum speed possible for the wheel attached to the
        motor shaft, given the motor specs, gear ratio and wheel circumference.

        This value depends on the correct :data:`wheel_circumference`
        value being set.
        """

        return self.max_rpm / 60.0 * self.wheel_circumference



from .adc_base import ADCBase


class Potentiometer(ADCBase):
    """Encapsulates the behaviour of a potentiometer.

    A potentiometer is a three-terminal resistor with a sliding or rotating contact that forms an
    adjustable voltage divider. The component is used for measuring the electric potential (voltage)
    between the two 'end' terminals. If only two of the terminals are used, one end and the wiper,
    it acts as a variable resistor or rheostat. Potentiometers are commonly used to control
    electrical devices such as volume controls on audio equipment.

    Uses an Analog-to-Digital Converter (ADC) to turn the analog reading from the sensor
    into a digital value.

    :param str port_name: The ID for the port to which this component is connected
    :param str number_of_samples: Amount of sensor samples used to report a :attr:`~.Potentiometer.position`. Defaults to 1.
    :param str name: Component name, defaults to `potentiometer`. Used to access this component when added to a :class:`pitop.Pitop` object.
    """

    def __init__(
        self, port_name, pin_number=1, name="potentiometer", number_of_samples=1
    ):
        ADCBase.__init__(
            self,
            port_name=port_name,
            pin_number=pin_number,
            name=name,
            number_of_samples=number_of_samples,
        )

    @property
    def own_state(self):
        return {
            "position": lambda: self.position,
        }

    @property
    def position(self):
        """Get the current reading from the sensor.

        :return: A value representing the potential difference (voltage)
            from 0 to 999.
        :rtype: float
        """
        return self.read()

    @property
    def value(self):
        """Get a simple binary value based on a reading from the device.

        :return: 1 if the sensor is detecting a potential difference
            (voltage), 0 otherwise
        :rtype: integer
        """
        if self.position > 0:
            return 1
        else:
            return 0



class UltrasonicRegisterTypes:
    CONFIG = 0
    DATA = 1


UltrasonicA1 = {
    UltrasonicRegisterTypes.CONFIG: 0x0C,
    UltrasonicRegisterTypes.DATA: 0x0E,
}


UltrasonicA3 = {
    UltrasonicRegisterTypes.CONFIG: 0x0D,
    UltrasonicRegisterTypes.DATA: 0x0F,
}

UltrasonicRegisters = {
    "A1": UltrasonicA1,
    "A3": UltrasonicA3,
}

UltrasonicConfigSettings = {
    "A1": 0xA1,
    "A3": 0xA3,
}
